
/*
 * VServer Context Quota
 * basic tools for quota hash manipulation
 *
 * mquota.c
 *
 * (C) 2003 Herbert Ptzl <herbert@13thfloor.at>
 *
 * Changelog:
 *
 *   0.01  - quota domain utility
 *   0.02  - enable/disable quota on path
 *	   - fixed bug in format request
 *   0.03  - add/remove quota hashes
 *	   - cleanup of tools for release
 *   0.04  - get/set quota hash disk limits
 *   0.05  - dlimits interface changed
 *
 * This file is part of vquota-tools.
 *
 * vquota-tools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * vquota-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with cquota-tools; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#define VERSION "V0.05"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include <unistd.h>
#include <libgen.h>

#include "quota.h"


static	char *	mnt_path = "/";
static	char *	quota_file = "/";
static	char *	disk_limits = "*,*,-1,-1";

static	char	err_flag = 0;

static	char    opt_verbose = 0;
static	char    opt_setdlim = 0;

static	enum	{
 	    IDENT_MQUOTA=0,
 	    IDENT_MQON=1,
 	    IDENT_MQOFF,
 	    IDENT_MQSYNC,
 	    IDENT_CQHADD,
 	    IDENT_CQHREM,
 	    IDENT_CQDLIM,
	}   	identity = IDENT_MQUOTA;

static	char *	cmd_name;

unsigned int	quota_type = -1;
unsigned int	quota_format = 0;
unsigned int	quota_ctx = 0;


int 	do_dlimctl(int cmd, int ctx, char *mnt, char *limits)
{
    	int ret;
	struct dli_limits dli = { 0,0,0,0,0 };
	
	if (cmd == Q_SET_DLIMIT) {
	    sscanf(limits, "%u,%u,%u,%u,%u",
	    	&dli.dli_inodes_used, &dli.dli_inodes_total,
	    	&dli.dli_blocks_used, &dli.dli_blocks_total,
		&dli.dli_reserved);
	}
	
	ret = quotactl(QCMD(cmd, 0), mnt, ctx, (void *)&dli);
	if (ret) {
	    if (opt_verbose)
	    	fprintf(stderr, "dlimctl failed: %s\n", strerror(errno));
	} else {
	    fprintf(stdout, "inodes:\t%u/%u\n",
	    	dli.dli_inodes_used, dli.dli_inodes_total);
	    fprintf(stdout, "blocks:\t%u/%u (%u%%)\n",
	    	dli.dli_blocks_used, dli.dli_blocks_total,
		dli.dli_reserved);
	}
	return ret;
}


int 	do_quotactl(int cmd, int type, char *mnt, char *file, int format)
{
    	int ret;
	
	if (opt_verbose) {
	    switch (cmd) {
	    case Q_QUOTAON:
	    case Q_QUOTAOFF:
	    	fprintf(stderr, "%sabling quota [%d] on %s [%s,%d] ... ", 
		    (cmd == Q_QUOTAON)?"en":"dis", type, mnt, file, format);
		break;
	    case Q_SYNC:
	    	fprintf(stderr, "syncing quota [%d] for %s ... ", type, mnt);
	    	break;
	    case Q_ADD_DQHASH:
	    case Q_REM_DQHASH:
	    	fprintf(stderr, "%sing quota hash for %s ... ", 
		    (cmd == Q_ADD_DQHASH)?"add":"remov", mnt);
	    	break;
	    }
	}
	ret = quotactl(QCMD(cmd, type), mnt, format, (void *)file);
	if (ret) {
	    if (opt_verbose)
	    	fprintf(stderr, "failed: %s\n", strerror(errno));
	    else
		fprintf(stderr, "quotactl on %s failed: %s\n", 
		    mnt, strerror(errno));
	}
	else if (opt_verbose)
	    fprintf(stderr, "succeeded.\n");
    	return ret;
}


int	main(int argc, char *argv[])
{
        extern int optind;
        extern char *optarg;
        char c, ret=0;

        cmd_name = basename(argv[0]);
	if (strcmp(cmd_name, "mqon") == 0)
		identity = IDENT_MQON;
	if (strcmp(cmd_name, "mqoff") == 0)
		identity = IDENT_MQOFF;
	if (strcmp(cmd_name, "mqsync") == 0)
		identity = IDENT_MQSYNC;
	if (strcmp(cmd_name, "cqhadd") == 0)
		identity = IDENT_CQHADD;
	if (strcmp(cmd_name, "cqhrem") == 0)
		identity = IDENT_CQHREM;
	if (strcmp(cmd_name, "cqdlim") == 0)
		identity = IDENT_CQDLIM;

        while ((c = getopt(argc, argv, "hvt:f:F:x:S:")) != EOF) {
            switch (c) {
            case 'h':
                fprintf(stderr,
                    "This is %s " VERSION "\n"
                    "options are:\n"
                    "-h        print this help message\n"
                    "-v        verbose output\n"
		    "%s%s%s", cmd_name,
		    (identity && (identity < IDENT_CQHADD))?
                    "-t <num>  quota type number\n"
		    :"",
		    (identity && (identity < IDENT_MQSYNC))?
                    "-f <path> quota file path\n"
                    "-F <num>  quota file format\n"
		    :"",
		    (identity && (identity == IDENT_CQDLIM))?
                    "-x <num>  context id\n"
                    "-S <vals> current/limit values\n"
		    :"");
                exit(0);
                break;
            case 'v':
                opt_verbose = 1;
                break;
            case 't':
                quota_type = atoi(optarg);
                break;
            case 'f':
                quota_file = optarg;
                break;
            case 'F':
                quota_format = atoi(optarg);
                break;
            case 'x':
                quota_ctx = atoi(optarg);
                break;
            case 'S':
                disk_limits = optarg;
		opt_setdlim = 1;
                break;
            case '?':
            default:
                err_flag++;
                break;
            }
        }
        if (err_flag || (optind >= argc)) {
            fprintf(stderr, 
                "Usage: %s -[hv]%s%s%s path\n"
                "%s -h for help.\n",
                cmd_name, 
		(identity && (identity < IDENT_CQHADD))?
		" [-t <num>]":"",
 		(identity && (identity < IDENT_MQSYNC))?
		" [-f <path>] [-F <num>]":"",
 		(identity && (identity == IDENT_CQDLIM))?
		" [-x <ctx>] [-S <values>]":"", cmd_name);
            exit(2);
        }

    	mnt_path = argv[optind];

    	switch (identity) {
	case IDENT_MQUOTA:
            fprintf(stderr, 
                "%s is a multicall binary\n",
		cmd_name);
	    exit(3);
	    
	case IDENT_MQON:
	    ret = do_quotactl(Q_QUOTAON, quota_type, 
	    	mnt_path, quota_file, quota_format);
	    break;
	case IDENT_MQOFF:
	    ret = do_quotactl(Q_QUOTAOFF, quota_type, 
	    	mnt_path, quota_file, quota_format);
	    break;
	case IDENT_MQSYNC:
	    ret = do_quotactl(Q_SYNC, quota_type, mnt_path, NULL, 0);
	    break;
	case IDENT_CQHADD:
	    ret = do_quotactl(Q_ADD_DQHASH, 0, mnt_path, NULL, 0);
	    break;
	case IDENT_CQHREM:
	    ret = do_quotactl(Q_REM_DQHASH, 0, mnt_path, NULL, 0);
	    break;
	case IDENT_CQDLIM:
	    ret = do_dlimctl(opt_setdlim?Q_SET_DLIMIT:Q_GET_DLIMIT, 
	    	quota_ctx, mnt_path, disk_limits);
	    break;
	}
	
	exit((ret)?1:0);
}

